#pragma once //microsoft-specific

/**
* This file is a filter for asserts. 
* Include this file to a class instead of assert.h to make sure the class' assert statements only run if it's debug mode. 
* Usage: Include this file only in .cpp files, not in header files. 
* 
* Strategy: 
* The code in this project generally contains a lot of asserts because they are easier to trace than crashes,
* and by having this flag defined they will not slow down release version.
* Asserts that should be active also for release mode and give player feedback should exist only in main.cpp
*/

// This code makes sure the NDEBUG (no debug) flag is the opposite of what the _DEBUG flag is in compiler build setup (debug/release)
// if the _DEBUG flag is set, the NDEBUG should not be set. if the _DEBUG is NOT set, then the NDEBUG should be set.
// The NDEBUG flag is then read and used by cassert
#ifdef _DEBUG
	#undef NDEBUG
#else
	#define NDEBUG
#endif

#include <cassert>