#pragma once //microsoft-specific

#include "Globals.h" // StateKey, Action
#include <vector>

class Timer; // forward declaration to speed up compilation

/**
 * The base class for states. 
 * Usage: start by calling BeginState(), continously call Update()
 * with actions that the state should/can react to and then
 * EndState() when finishing.
 */
class BaseState
{
public:
								BaseState(StateKey& stateKey, Timer& timer);
	virtual						~BaseState();

	// Stuff that should happen before updating starts.
	// By default, nothing happens
	virtual void				BeginState();

	// Stuff that should happen after updating ended, before next state or quit
	// By default, nothing happens
	virtual void				EndState();

	// Run & update the current state. May react to actions given.
	virtual void				Update(std::vector<Action*>& actionsRef) = 0; 

	// Tells if the state wants to go to a new state by returning one other than STATE_NONE
	// By default always returns STATE_NONE
	virtual	StateKey			DesiredStateChange() const;

	// Gets the state key associated with this class
	StateKey					GetStateKey() const;

protected:
	Timer&						m_timerRef;

private:

	// Avoid assign of this class, it should not be used in that way
	BaseState&					operator=(const BaseState& ); 

	StateKey					m_stateKey; // key that identifies this state. private since it must be consistent with what the statefactory thought when choosing this class; no de-coupling
};
