#include "BonusLife.h"
#include "Globals.h" // PADDING, SCREEN_WIDTH, SCREEN_HEIGHT
#include "ImageConfig.h" //IMG_ID
#include "GraphicsSystem.h"

// Game is using a lot of enums instead of defines in order not to clutter the global namespace, overwrite something in other libraries, be easier to re-scope and debug
enum {  
		COUNTER_VALUE_TO_KEEP_BONUS_LIFE = 400,
		COUNTER_VALUE_LEFT_TO_BE_BLINKING = 150,
		END_SCREEN_DISTANCE_X = 200,
		BONUSLIFE_SPAWN_INTERVAL = 100, 
		COUNTER_VALUE_TO_SPAWN_BONUS_LIFE = 300, 
		MAX_SPAWN_TRIES = 10, 
		MIN_SPAWN_TIME = 5
};


BonusLife::BonusLife(const Galaxy& galaxyRef)
:RoundEntity(GraphicsSystem::GetSingleton()->GetImage(IMG_ID_WRENCH), galaxyRef)
{
	SetSpawnInterval(BONUSLIFE_SPAWN_INTERVAL);
}

BonusLife::~BonusLife()
{
}
 
void BonusLife::ResetBeforeLevel()
{
	Unspawn();
	SetSpawnCounter(COUNTER_VALUE_TO_SPAWN_BONUS_LIFE);
	SetOpacity(1.0f);
	m_aliveCounter = 0.0f;
}

void BonusLife::Update(float elapsedTime)
{
	if (GetSpawnCounter() > 0.0f) // has not yet spawned
	{
		SetSpawnCounter(GetSpawnCounter() - GAME_SPEED * elapsedTime);

		if (GetSpawnCounter() <= 0.0f)
		{
			TrySpawn();
			if (IsAlive()) // If it succeeded to spawn
			{
				m_aliveCounter = COUNTER_VALUE_TO_KEEP_BONUS_LIFE;
			} // Else the normal counter above will have been reset
		}
	} 
	if (IsAlive())
	{	
		m_aliveCounter -= GAME_SPEED * elapsedTime;

		if (m_aliveCounter <= 0.0f)
			Unspawn(); // dies by self if it didnt die already
	} 
}

void BonusLife::Draw()
{
	if (!IsAlive()) return;

	if (m_aliveCounter < COUNTER_VALUE_LEFT_TO_BE_BLINKING)
	{		
		if (m_lastCounterMark - m_aliveCounter >= 15.0f ) // every 15th counter step
		{
			if (GetOpacity() == 1.0f) // toggle opacity
				SetOpacity(0.0f);
			else 
				SetOpacity(1.0f);

			m_lastCounterMark = m_aliveCounter;
		}
	} 
	else // keep blink counter mark up to date in case its not time to start blink yet
	{
		m_lastCounterMark = m_aliveCounter;
	}

	Entity::Draw();
}

void BonusLife::TrySpawn()
{
	RoundEntity::TrySpawn(MAX_SPAWN_TRIES, MIN_SPAWN_TIME,  (SCREEN_WIDTH - GetWidth() - END_SCREEN_DISTANCE_X - 2 * PADDING), (SCREEN_HEIGHT - GetHeight() - 2 * PADDING), (float) PADDING, (float) PADDING);
}
