#include "FadingMusicText.h"
#include "SoundSystem.h"
#include "AssertButOnlyInDebug.h"

enum { 
		FADING_TEXT_POS_Y = 5 
};

FadingMusicText::FadingMusicText()
:FadingText(false, FADING_TEXT_POS_Y, LEFT)
{
}


FadingMusicText::~FadingMusicText()
{
}

void FadingMusicText::ResetBeforeLevel(int level)
{
	assert(level >= 0 && "Level cannot be negative number");
	FadingText::Reset();
	SetMusicTitle(level); // What SHOULD be the next music if it plays
}

void FadingMusicText::Draw()
{
	// engine usage optimization: saving local cache of playing status and updating once per level instead of keep polling soundsystem for info
	if (SoundSystem::GetSingleton()->IsMusicPlaying())
		FadingText::Draw();

}

void FadingMusicText::SetMusicTitle(int level)
{
	const char* musicURL = SoundSystem::GetSingleton()->GetMusicURL(level-1);  // Example1: reso/vor.ogg
									// Example2: arg.ogg

	int lastBackSlashPosInURL = -1;  // default: none, outside string                    
	int lastDotPos = strlen(musicURL);	// default: after last sign, outside string
	for (Uint8 i = 0; i < strlen(musicURL); i++) 
	{
		if (musicURL[i] == '/')
		{ 
			lastBackSlashPosInURL = i;	
		}
		if(musicURL[i] == '.')
		{
			lastDotPos = i;     
		}
	}
	
	
	
	// adding "music"
	char title[TEXT_BUFFER];      

	const char* prefix = "Music: ";
	strcpy_s(title, prefix); // microsoft-specific that does not cause buffer overrun
	
	int startPosInTargetString = strlen(prefix);
	int posInURL;
	
	int nrOfChars = lastDotPos - 1 - lastBackSlashPosInURL; 

	// adding song title
	for (int posInTargetString = startPosInTargetString; posInTargetString < (nrOfChars + startPosInTargetString); posInTargetString++)
	{
		posInURL = lastBackSlashPosInURL+1+(posInTargetString-startPosInTargetString);
		title[posInTargetString] = musicURL[posInURL];
	}
	title[nrOfChars + startPosInTargetString] = '\0'; // null-terminating char
	
	FadingText::SetText(title);  // Copies

	// char and prefix are local and should be deallocated automatically
}