#include "FadingText.h"
#include "GraphicsSystem.h"
#include "ScreenManager.h"
#include "AssertButOnlyInDebug.h"


enum { 
	ONE_FADE_LENGTH = 100 
};

enum { 
		WHITE_TEXT_R_VALUE = 255,
		WHITE_TEXT_G_VALUE = 255,
		WHITE_TEXT_B_VALUE = 255
};

FadingText::FadingText(bool bIsLooping, float posY, XAlignment xAlign)
{
	assert((posY > 0.0f && posY < SCREEN_HEIGHT) && "The text must be visible!");
	m_bIsLooping = bIsLooping;
	m_xAlign = xAlign;
	m_posY = posY;
	Reset();
}

FadingText::~FadingText()
{
}

void FadingText::Reset()
{
	m_animationCounter = 0;
}
void FadingText::Draw()
{
	if (m_animationCounter < 2 * ONE_FADE_LENGTH)   
	{
		float opacity = (float) (m_animationCounter%ONE_FADE_LENGTH);
		opacity/=ONE_FADE_LENGTH; 

		assert(m_text && "FadingText::Draw(): Cannot draw fading text: the text has not been set!");

		if (m_animationCounter >= ONE_FADE_LENGTH) 
			opacity = 1.0f - opacity; 

		//
		GraphicsSystem::GetSingleton()->DrawTextOntoScreenBuffer(m_text, m_posY, m_xAlign, WHITE_TEXT_R_VALUE, WHITE_TEXT_G_VALUE, WHITE_TEXT_B_VALUE, FONT_SIZE_SMALL, opacity); 
		
		// must be time controlled since counter is not connected to delta time
		m_animationCounter++; 
	}

	if (m_bIsLooping && m_animationCounter >= 2 * ONE_FADE_LENGTH)
		Reset();
}
void FadingText::SetText(const char* text)
{
	assert(text && "Trying to set fading animation text that is null");
	assert(strlen(text) > 0 && "Cannot set empty text!");
	
	strcpy_s(m_text, text);
}