#pragma once //microsoft-specific

#include <vector>

// forward declaration to speed up compilation
class ImageSprite;
class Rock;
class BonusLife;
class StarField;
class Entity; // cross reference
class ICollidableEntity; 
class IIntersectableEntity;

/**
 * Class represents the world, owning all rock and bonus life entities.
 * Everything about them is controlled through here. 
 * Handles collision detection against and between entities within the galaxy. 
 */
class Galaxy
{
public:
								Galaxy();
	~Galaxy();
	void 						Update(float elapsedTime, bool bIsStartOfLevel);
	void 						Draw();
	void 						ResetBeforeGame();

	// Changes background and re-sets entities
	void 						ResetBeforeLevel(int level);

	//-----------------------------------
	// Collision functions
	//-----------------------------------
	// Checks for collisions between objects that has been moving;
	// player vs bonuslife / rocks
	void 						HandleCollisions(ICollidableEntity& playerRef);

	// Checks if the incoming entity hit a rock (by calling intersect function
	// in the entity argument itself against each rock)
	// Call by the entities in their spawn functions to make sure 
	// they don't spawn and overlap something. 
	bool 						HasCollidedWithRock(const IIntersectableEntity& entityRef) const;

	//-------------------------
	// Functions for level events
	//-------------------------
	void 						StopSpawnRocks();
	void 						IncreaseRockSpeed();
	bool 						IsAllowedToSpawnRocks() const;


private:
	// Changes to corresponding background
	void 						ChangeBackground(int level);
	
	// Checks if the player has collected a bonus life
	void 						HandlePlayerVsBonusLifeCollisions(ICollidableEntity& playerRef);

	// Checks if the player has collided with a rock
	void 						HandlePlayerVsRockCollisions(ICollidableEntity& playerRef);

	// Avoid assign of this class, it should not be used in that way
	Galaxy& 					operator=(const Galaxy& );

	ImageSprite& 				m_imgSpriteRef; 
	std::vector<Rock*>			m_rocks;
	BonusLife*					m_bonusLife;
	StarField*					m_starField;
};