#include "GraphicsSystem.h"
#include "ImageSprite.h"
#include "ImageSubSystem.h"
#include "TextSubSystem.h"
#include "AssertButOnlyInDebug.h"

// Re-declaring to get handle for static instance 
GraphicsSystem* GraphicsSystem::ms_graphics = NULL; 

GraphicsSystem* GraphicsSystem::GetSingleton() 
{
	if (ms_graphics == NULL) 
		ms_graphics = new GraphicsSystem();

	return ms_graphics;
}

GraphicsSystem::GraphicsSystem()
{
	m_imageSubSystem = NULL;
	m_textSubSystem = NULL;	
}

GraphicsSystem::~GraphicsSystem()
{
	printf("INFO: Destroying GraphicsSystem\n");

	if (m_imageSubSystem != NULL)
	{
		delete m_imageSubSystem;
		m_imageSubSystem = NULL;
	}

	if(m_textSubSystem != NULL)
	{
		delete m_textSubSystem;	
		m_textSubSystem = NULL;
	}

	SDL_Quit(); 
}

void GraphicsSystem::Destroy() 
{
	if (ms_graphics != NULL) 
	{
		delete ms_graphics;
		ms_graphics = NULL;
	}
}


bool GraphicsSystem::Init()
{
	m_textSubSystem = new TextSubSystem();
	m_imageSubSystem = new ImageSubSystem();

	if (!InitGameWindow() ||
		!m_textSubSystem->Init()  || 
		!m_imageSubSystem->Init())
		return false;
	return true;
}

bool GraphicsSystem::InitGameWindow()
{	
	_putenv("SDL_VIDEO_CENTERED=1"); 

    if ( SDL_Init(SDL_INIT_VIDEO ) )
    {
		printf("ERROR: %s: Failed to initialize SDL video: %s\n", __FUNCTION__, SDL_GetError()); //microsoft-specific
		assert(false && "GraphicsSystem::InitGameWindow(): Failed to initialize SDL video. Check log for more details.");		
        return false;
    }
    printf("INFO: Successfully initialized SDL\n");

	// hides cursor so user wont try use it
	SDL_ShowCursor(SDL_DISABLE);

	// Open GL 
    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	// Frame title
    SDL_WM_SetCaption("Star - (C) Marie Mejerwall 2011-2014 - www.ventria.se", NULL);
    SDL_WM_SetIcon(SDL_LoadBMP("resources/textures/StarIcon.bmp"), NULL);

    SDL_Surface* screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 0, SDL_OPENGL);
    if (screen == NULL)
    {
		printf("ERROR: %s: Failed to create window: SDL_SetVideoMode error\n", __FUNCTION__); // microsoft-specific
		assert(false && "GraphicsSystem::InitGameWindow(): Failed to create window: SDL_SetVideoMode error");
        return false;
    }
	printf("INFO: Successfully created game window\n");

	// Init OpenGL
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glDisable(GL_DEPTH);
    glDisable(GL_DEPTH_TEST);
	printf("INFO: Successfully initialized OpenGL\n");

	return true;
}


ImageSprite& GraphicsSystem::GetImage(IMG_ID id)
{
	return m_imageSubSystem->GetImage(id);
}

void GraphicsSystem::DrawTextOntoScreenBuffer(const char *textToDisplay, GLfloat posY, XAlignment alignX, Uint8 textRed, Uint8 textGreen, Uint8 textBlue, FontSize fontSize, float opacity)
{
	m_textSubSystem->DrawTextOntoScreenBuffer(textToDisplay, posY, alignX, textRed, textGreen, textBlue, fontSize, opacity);
}

void GraphicsSystem::ClearScreenBufferWithBlack()
{
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
}


void GraphicsSystem::ShowBufferedScreenOnDisplay()
{
    glFlush(); // Executes openGL commands left to manupulate next frame buffer
    SDL_GL_SwapBuffers(); // Updates screen by switching to the next screen frame buffer
}

std::vector<const char*> GraphicsSystem::ReadTextFromFile(TEXT_ID id)
{
	return m_textSubSystem->ReadTextFromFile(id);
}