#pragma once //microsoft-specific

/**
* Configuration file for graphical resources.
*
* Each graphical sprite has an enum tied to it.
* The ImageSubSystem resource management as well as various classes in the game
* uses these enums for retrieving an imagesprite.
*
* An X-macro is then used to map the resource (enum) with its path string. 
* 
* Macros can be blunt and overwrite anything, don't have types, ignore scoping, 
* be hard to debug etc but this one is neat and I can stick to unique naming.
* 
* I consider it as good practice that the classes request resource by this ENUM and can't just type anything at any time. 
* Mistakes are usually found early at compile-time.
*/
#define IMG_ID_AND_PATH_TABLE \
X(IMG_ID_GALAXY_BG1,"resources/textures/bg1.jpg") \
X(IMG_ID_GALAXY_BG2, "resources/textures/bg2.jpg") \
X(IMG_ID_GALAXY_BG3, "resources/textures/bg3.jpg") \
X(IMG_ID_GALAXY_BG4, "resources/textures/bg4.jpg") \
X(IMG_ID_GALAXY_BG5, "resources/textures/bg5.jpg") \
X(IMG_ID_GALAXY_BG6, "resources/textures/bg6.jpg") \
X(IMG_ID_ROCK1, "resources/textures/rock1.png") \
X(IMG_ID_ROCK2, "resources/textures/rock2.png") \
X(IMG_ID_ROCK3, "resources/textures/rock3.png") \
X(IMG_ID_ROCK4, "resources/textures/rock4.png") \
X(IMG_ID_SHIP, "resources/textures/ship.png") \
X(IMG_ID_SMALL_SHIP, "resources/textures/ship_ui.png") \
X(IMG_ID_FIRE, "resources/textures/fire.png") \
X(IMG_ID_KEYS, "resources/textures/keys.png") \
X(IMG_ID_STAR_FIELD, "resources/textures/starfield.png") \
X(IMG_ID_BG_STAR_FIELD, "resources/textures/bg_starfield.png") \
X(IMG_ID_GLOWING_STARS1, "resources/textures/glowing_stars1.png") \
X(IMG_ID_GLOWING_STARS2, "resources/textures/glowing_stars2.png") \
X(IMG_ID_GLOWING_STARS3, "resources/textures/glowing_stars3.png") \
X(IMG_ID_LOGO, "resources/textures/logo.png") \
X(IMG_ID_MENU, "resources/textures/menu.png") \
X(IMG_ID_METER_STRIP, "resources/textures/meter_strip.png") \
X(IMG_ID_WRENCH, "resources/textures/wrench.png") \
X(IMG_ID_WRENCH_UI, "resources/textures/wrench_ui.png")

// X-macro for mapping enums for imagesprites together with their paths as strings
// Defining the enums below. The path string definition is in ImageSubSystem.
#define X(a, b) a,
	enum IMG_ID {
	  IMG_ID_AND_PATH_TABLE
	};
#undef X

// Some textures has hitboxes that are smaller than the image dimensions
enum ROCK_DIAMETER { 
	SPRITE_DIAMETER_ROCK1 = 32,
	SPRITE_DIAMETER_ROCK2 = 45,
	SPRITE_DIAMETER_ROCK3 = 64,
	SPRITE_DIAMETER_ROCK4 = 64
};
