#pragma once //microsoft-specific

#include "SDL_opengl.h" // GLfloat, GLuint

/**
* Encapsulates a texture
*/
class ImageSprite
{
public:
								ImageSprite(const char* filePath);

	// Cleans upp. Deletes internal member m_OGL_Texture
								~ImageSprite();

	//Loads address of image to OGL surface. Returns false if unsuccessful. 
	bool 						Load();

	// Puts the texture in the screen buffer 
	void 						Draw();

	int 						GetHeight() const;
	int 						GetWidth() const;
	float 						GetOpacity() const;
	float 						GetScreenX() const;
	float 						GetScreenY() const;
	void 						SetOpacity(float opacity);
	void 						SetWidth(int width);
	void 						SetHeight(int height);
	void 						SetScreenX(float screenX);
	void 						SetScreenY(float screenY);

	// Origo represents the center pivot point of the texture.
	// It's relative to m_screenX and m_screenY (has its starting point in them), not to world space
	// Passing half the texture's width and height will put the origo in the center of the texture
	void 						SetOrigo(float origoX, float origoY);

	// Sets the rotation around the origo that this texture should have during the following calls to draw.
	void 						SetRotationDegree(float rotationDegree);

private:
	GLfloat 					m_screenX;
    GLfloat 					m_screenY;
	GLfloat						m_opacity;
	int 						m_width;
	int 						m_height;
	char 						m_fileName[256];
	GLfloat 					m_redHue;
    GLfloat 					m_greenHue;
    GLfloat 					m_blueHue;
	bool 						m_bSmooth;
    GLfloat 					m_rotationDegree;
	GLuint 						m_OGL_Texture;
	GLfloat 					m_origoX;
	GLfloat 					m_origoY;
};
