#include "Input.h"
#include "SDL.h"

Input::Input()
{
}

Input::~Input()
{
}

std::vector<Action*> Input::GetNewActions() const
{
	std::vector<Action*> actions;
	SDL_Event currentSDLAction;

	// Takes care of queued events, runs until there are no more events queued or user calls quit.
	// Will overwrite states set before in the same loop
	while (SDL_PollEvent(&currentSDLAction)) 
    {
        switch (currentSDLAction.type)
		{
            case SDL_QUIT:
			{
				actions.push_back(new Action(A_QUIT));
				return actions;	 // no further processing necessary
			}

            case SDL_KEYDOWN:
			{
                switch( currentSDLAction.key.keysym.sym ) 
                {
                    case SDLK_ESCAPE:
                    {
						actions.push_back(new Action(A_QUIT));
						return actions; // will not check for further events
						break; 
                    }
					case SDLK_p:
					{
						actions.push_back(new Action(A_PLAY));
						break;
					}
					case SDLK_PAUSE:
					{
						actions.push_back(new Action(A_PAUSE));
						break;
					}
					case SDLK_q:
					{
						actions.push_back(new Action(A_QUIT));
						return actions; // will not check for further events
					}
					case SDLK_c:
					{
						actions.push_back(new Action(A_CREDITS));
						break;
					}
					case SDLK_i:
					{
						actions.push_back(new Action(A_INSTRUCTIONS));
						break;
					}
					case SDLK_RETURN:
					{
						actions.push_back(new Action(A_DEFAULT)); break;
					}
                    default:
					{
						actions.push_back(new Action(A_NONE)); // signaling that something was pressed even though not handled
                        break;
					}
                }
				break;
			}
			default:
				break;
        }	
	}

	Uint8 *keystate = SDL_GetKeyState(NULL); 

	// Checking if keys are still pressed, even if there were no "new events" 
	if (keystate[SDLK_UP] && !keystate[SDLK_DOWN])	
		actions.push_back(new Action(A_UP));
	else if (keystate[SDLK_DOWN] && !keystate[SDLK_UP])
		actions.push_back(new Action(A_DOWN));


	if (keystate[SDLK_RIGHT] && !keystate[SDLK_LEFT])
		actions.push_back(new Action(A_RIGHT));
	else if (keystate[SDLK_LEFT] && !keystate[SDLK_RIGHT])
		actions.push_back(new Action(A_LEFT));

	return actions;
}
