#include "IntroState.h"
#include "Timer.h"
#include "ScreenManager.h"
#include <stdio.h>

enum { 
			SCREENS_DURATION_IN_MS = 2000 
};

IntroState::IntroState(StateKey stateKey, Timer& timer):
BaseState(stateKey, timer)
{
	m_currentSubState = INTRO_SUB_STATE_PRESENTATION;
}
void IntroState::BeginState()
{
	printf("INFO: Beginning IntroState\n");
	m_timerRef.SetNewTimerAndGetElapsedTime();
	m_updateTimer = 0.0f;
}
void IntroState::Update(std::vector<Action*>& actionsRef)
{
	bool bKeyWasPressed = !actionsRef.empty();

	switch(m_currentSubState)
	{
		case INTRO_SUB_STATE_PRESENTATION: UpdatePresentationSubState(bKeyWasPressed); break;
		case INTRO_SUB_STATE_LOGO: UpdateLogoSubState(bKeyWasPressed); break;
		case INTRO_SUB_STATE_STORY: UpdateStorySubState(bKeyWasPressed); break;
		default: break;
	}
					
	m_timerRef.WaitSeconds(0.01f);	
}

StateKey IntroState::DesiredStateChange() const
{
	if (m_currentSubState == INTRO_OVER) 
		return STATE_MENU;

	return STATE_NONE;
}

void IntroState::UpdatePresentationSubState(bool bKeyWasPressed)
{
	ScreenManager::GetSingleton()->ShowTextOnScreen("Marie Mejerwall", "presents", false);

	m_updateTimer += m_timerRef.SetNewTimerAndGetElapsedTime();

	if (bKeyWasPressed || m_updateTimer > SCREENS_DURATION_IN_MS) // user can abort, either case it will autofade
		MoveToNextSubState();

}
void IntroState::UpdateLogoSubState(bool bKeyWasPressed)
{
	ScreenManager::GetSingleton()->ShowScreen(SCREEN_LOGO); 

	m_updateTimer += m_timerRef.SetNewTimerAndGetElapsedTime();

	if (bKeyWasPressed || m_updateTimer > SCREENS_DURATION_IN_MS) // user can abort, either case it will autofade
		MoveToNextSubState();

}
void IntroState::UpdateStorySubState(bool bKeyWasPressed)
{
	ScreenManager::GetSingleton()->ShowScreen(SCREEN_STORY); 

	if (bKeyWasPressed) // user must press and indicate they saw it
		MoveToNextSubState();
}

void IntroState::MoveToNextSubState()
{
	m_updateTimer = 0.0f;
	m_timerRef.SetNewTimerAndGetElapsedTime();

	if (m_currentSubState < INTRO_OVER)
		m_currentSubState = INTRO_SUB_STATE(m_currentSubState + 1); // next
}
void IntroState::EndState()
{
	printf("INFO: Ending IntroState\n");
}