#pragma once //microsoft-specific

#include "BaseState.h"

/**
* Customized state that shows a fixed order series of screens.
* Transition between these occur automatically with time and user input.
*/
class IntroState :
	public BaseState
{
public:
								IntroState(StateKey stateKey, Timer& timer);

	// Implementation. Updates the current substate
	void 						Update(std::vector<Action*>& actionsRef); 

	 // Override. Will return STATE_MENU once all substates are finished updating.
	StateKey 					DesiredStateChange() const;

	// Override. Resets timer
	void 						BeginState();

	// Override
	void 						EndState();

private:

	// Substates that this state goes through in order
	enum						INTRO_SUB_STATE {
									INTRO_SUB_STATE_PRESENTATION,
									INTRO_SUB_STATE_LOGO,
									INTRO_SUB_STATE_STORY,
									INTRO_OVER
	};

	void						MoveToNextSubState();
	void 						UpdatePresentationSubState(bool bKeyWasPressed);
	void 						UpdateLogoSubState(bool bKeyWasPressed);
	void 						UpdateStorySubState(bool bKeyWasPressed);

	enum INTRO_SUB_STATE		m_currentSubState;
	bool 						m_bIntroOver;
	float 						m_updateTimer;
};

