#pragma once //microsoft-specific

#include "BaseState.h"

// forward declaration to speed up compilation
class Galaxy;
class Player;
class LevelManager;
class HUD;

/**
 * The play loop. Active class controlling game & level events.
 * Call and update continously to run the game one update & draw frame in its current state. 
 * Will return a desired changed state on win and/or loss. 
 **/
class PlayState: public BaseState
{
public:
								PlayState(StateKey stateKey, Timer& timer);
								~PlayState();

	// Override. Resets everything to it's initial state
	void 						BeginState();

	 // Override. Quickly fades out any music currently playing
	void 						EndState();

	// Runs the game in its current SubState, level by level
	// For each level, it resets everything, shows countdownscreen, 
	// plays new music and increases rock speed
	// It then plays the level until the level counter has ended. 
	// For every count (update), it updates the HUD, player, world, objects everything,
	// draws everything and shows it on screen. 
	// At the end of a level, it stops spawn rocks and fades out music.
	// After each level, it shows animation where ship flies out of screen.
	// Handles lose, win and quit according to actions and game events
	// Handles pausing through directing to the right substate
	void 						Update(std::vector<Action*>& actionsRef); 

	// Override. Will call for transition MenuState if the game is over
	StateKey 					DesiredStateChange() const; 

private:

	// Show play startscreen
	void 						ShowCountDownScreen(const char* levelText);

	void 						ResetBeforeGame();
	
	void 						ResetBeforeLevel();

	// Flies ship out of the picture
	void 						ShowShipFlyOutAnimation();
	
	void 						EnterPause();
	void						ExitPause();
	void 						HandlePauseSubState(std::vector<Action*>& actions);
	void 						HandlePlaySubState(std::vector<Action*>& actions);

	void 						EndLevel();
	void 						IncreaseLevel();
	void 						EndGame();
	void 						WinGame();

	Galaxy* 					m_galaxy;
	Player* 					m_player;
	LevelManager*				m_levelManager;
	HUD*						m_HUDManager;

	bool						m_bGameOver;
	bool						m_bIsGamePaused;
};
