#include "Rock.h"
#include <stdlib.h> // using rand
#include "Globals.h" // SCREEN_WIDTH
#include "ImageSprite.h"
#include "AssertButOnlyInDebug.h"

enum { 	ROCK_MIN_RESPAWN_TIME = 15,
		ROCK_MAX_RESPAWN_TIME = 150,
		MIN_DISTANCE_BETWEEN_ROCKS = 85,
		ROCK_MAX_TRY_RESPAWN_TIMES = 7,
		ROCK_START_SPEED = 5,
		MAX_ROCKS_TRY_SPAWNS_PER_UPDATE = 20,
		ROCK_MAX_ROTATION_SPEED_DIFFERENCE = 2
};

const float ROCK_MIN_ROTATION_SPEED = 0.4f;

// same variables for all rocks, each don't need separate variable
float Rock::m_rockSpeed; 
bool Rock::m_bIsAllowedToSpawn; 
short Rock::m_nrRemainingRockTrySpawnAttempts;

Rock::Rock(ImageSprite& imageSpriteRef, int hitBoxDiameter, Galaxy& galaxyRef)
:RoundEntity(imageSpriteRef, galaxyRef), 
m_rotationDegree(0),
m_rotationSpeed(ROCK_MIN_ROTATION_SPEED + rand()% ROCK_MAX_ROTATION_SPEED_DIFFERENCE) 
{
	assert(hitBoxDiameter >= 0 && "Hitbox cannot have 0 or negative as diameter");

	m_hitBoxDiameter = hitBoxDiameter;
	imageSpriteRef.SetOrigo((float) hitBoxDiameter * 0.5f, (float) hitBoxDiameter * 0.5f);
	SetMinDistance(MIN_DISTANCE_BETWEEN_ROCKS);
	SetSpawnInterval(ROCK_MAX_RESPAWN_TIME);
	ResetBeforeLevel(); // refactored
}

Rock::~Rock()
{
}

void Rock::ResetBeforeLevel()
{
	SetPositionX(SCREEN_WIDTH + (float)GetWidth());
	Unspawn();
	SetRandomSpawnTimer(0); 
}

void Rock::Update(float elapsedTime)
{
	if(!IsAlive())
	{
		if(GetSpawnCounter() > 0) // Rock is dead, lies left of screen, waits to spawn
			SetSpawnCounter(GetSpawnCounter() - (GAME_SPEED * elapsedTime));
		else if (GetSpawnCounter() <= 0.0f && m_nrRemainingRockTrySpawnAttempts > 0) // time to spawn, sets alive on 
		{
			m_nrRemainingRockTrySpawnAttempts--; // each attempt has several tries to find a free spot
			TrySpawn();
		}
		
	}
	else // Rock is alive and moving
	{
		SetPositionX( GetPositionX() - (Rock::m_rockSpeed * (GAME_SPEED * elapsedTime)) ); // moving 

		if (GetPositionX() < (0.0f - (float)GetWidth())) // if time to die, set spawn timer
		{
			Unspawn(); //die die
			SetRandomSpawnTimer(ROCK_MIN_RESPAWN_TIME);
		}
		m_rotationDegree += m_rotationSpeed * (GAME_SPEED * elapsedTime);
	}
	

}

void Rock::TrySpawn()
{
	if (!m_bIsAllowedToSpawn)
		return;

	RoundEntity::TrySpawn(ROCK_MAX_TRY_RESPAWN_TIMES, ROCK_MIN_RESPAWN_TIME, 0, (SCREEN_HEIGHT - PADDING - GetHeight()), (SCREEN_WIDTH + (float) GetWidth()), PADDING);
}

void Rock::Draw()
{
	if (!IsAlive())
		return;

	Entity::SetRotationDegree(m_rotationDegree);
	Entity::Draw();

}

void Rock::IncreaseRockSpeed()
{
	m_rockSpeed += 1.12f;
}
void Rock::ResetRockSpeed()
{
	m_rockSpeed = ROCK_START_SPEED;
}
void Rock::AllowSpawn()
{
	m_bIsAllowedToSpawn = true;
}
void Rock::StopSpawn()
{
	m_bIsAllowedToSpawn = false;
}
bool Rock::IsAllowedToSpawn()
{
	return m_bIsAllowedToSpawn;
}
float Rock::GetCenterX() const
{
	return GetPositionX() + m_hitBoxDiameter / 2.0f; 
}
float Rock::GetCenterY() const
{
	return GetPositionY() + m_hitBoxDiameter / 2.0f; 
}
float Rock::GetRadius() const
{
	return ((float) m_hitBoxDiameter) / 2.0f; 
}
void Rock::ResetNrOfTrySpawnAttempts(bool bShouldOnlyTrySpawnHalf)
{
	if (bShouldOnlyTrySpawnHalf)
		m_nrRemainingRockTrySpawnAttempts = MAX_ROCKS_TRY_SPAWNS_PER_UPDATE / 2; // rounded down
	else
		m_nrRemainingRockTrySpawnAttempts = MAX_ROCKS_TRY_SPAWNS_PER_UPDATE;
}