#include "RockFactory.h"
#include "GraphicsSystem.h"
#include "Galaxy.h"
#include "Rock.h"
#include "ImageConfig.h" // IMG_ID
#include "AssertButOnlyInDebug.h"



std::vector<Rock*> RockFactory::CreateNewMixedRocks(int nrOfRocks, Galaxy& owner)
{
	assert(nrOfRocks > 0 && "RockFactory::CreateNewMixedRocks(): must give a positive number of rocks");

	IMG_ID rockIds[] = { IMG_ID_ROCK1, IMG_ID_ROCK2, IMG_ID_ROCK3, IMG_ID_ROCK4 }; //sub range of enum to ensure their order
	size_t numRockIds = sizeof (rockIds) / sizeof (rockIds[0]);

	// re-listing the diameters here to ensure they are in order
	ROCK_DIAMETER rockDiameters[] = { SPRITE_DIAMETER_ROCK1, SPRITE_DIAMETER_ROCK2, SPRITE_DIAMETER_ROCK3, SPRITE_DIAMETER_ROCK4 };

	assert(sizeof (rockDiameters) / sizeof (rockDiameters[0]) == numRockIds && "RockFactory::CreateNewMixedRocks(): the number of rock types must be consistent");

	std::vector<Rock*> rocks;
	rocks.reserve(nrOfRocks);

	// Creates rocks with diffent textures
	for (int i = 0; i < nrOfRocks; i++)
	{
		IMG_ID currentRockId = rockIds[i % numRockIds];
		int currentDiameter = rockDiameters[i % numRockIds];
		rocks.push_back(new Rock(GraphicsSystem::GetSingleton()->GetImage(currentRockId), currentDiameter, owner));
	} 

	return rocks;
}