#include "RoundEntity.h"
#include <stdlib.h> // rand
#include "Galaxy.h"
#include "AssertButOnlyInDebug.h"

RoundEntity::RoundEntity(ImageSprite& imgSpriteRef, const Galaxy& galaxyRef):
Entity(imgSpriteRef),
m_minDistance(0.0f), // standard distance between this entity and others
m_galaxyRef(galaxyRef)
{
}


RoundEntity::~RoundEntity()
{
}

bool RoundEntity::IsIntersecting(const RoundEntity& otherRoundEntityRef) const
{
	// AABB check first

    if ((otherRoundEntityRef.GetCenterX() + otherRoundEntityRef.GetRadius() + m_minDistance < GetCenterX() - GetRadius())||
		(otherRoundEntityRef.GetCenterX() - otherRoundEntityRef.GetRadius() - m_minDistance > GetCenterX() + GetRadius())||
		(otherRoundEntityRef.GetCenterY() + otherRoundEntityRef.GetRadius() + m_minDistance < GetCenterY() - GetRadius())||
		(otherRoundEntityRef.GetCenterY() - otherRoundEntityRef.GetRadius() - m_minDistance > GetCenterY() + GetRadius()))
		return false;


	// Narrow check:
	//Pythagoras for triangles: lineLength[a]^2 + lineLength[b]^2 = hypotenusaLength[c]^2
	float actualDistanceSquare, distanceX, distanceY, minCenterDistanceSquare;

	distanceX = GetCenterX() - otherRoundEntityRef.GetCenterX();
	distanceY = GetCenterY() - otherRoundEntityRef.GetCenterY();
	actualDistanceSquare = distanceX*distanceX + distanceY*distanceY;
	minCenterDistanceSquare = (GetRadius() + otherRoundEntityRef.GetRadius() + m_minDistance) * (GetRadius() + otherRoundEntityRef.GetRadius()+ m_minDistance);

	if ( actualDistanceSquare < minCenterDistanceSquare ) 
		return true;

	return false;
}
bool RoundEntity::IsAlive() const
{ 
	return m_bIsAlive; 
}
void RoundEntity::Unspawn()
{
	m_bIsAlive = false;
}

void RoundEntity::TrySpawn(int maxTries, float minSpawnTime, int rangeX, int rangeY, float minPosX, float minPosY)
{
	m_bIsAlive = true;
	int tries = 0;
	
	// Tries to find free position to spawn for [maxTries] times
	do
	{
		if (rangeX <= 0) // minimum 1
			rangeX = 1;
		SetPositionX( minPosX + (float) (rand()%((int) rangeX)) ); // sets random position X. Not too far right or it will be hard to catch
		SetPositionY( minPosY + (float) (rand()%((int) rangeY)) ); // sets random position Y 
		tries++;

		if (tries == maxTries)
		{
			SetRandomSpawnTimer(minSpawnTime);
			m_bIsAlive = false; 
			break;
		}
	}
	while (m_galaxyRef.HasCollidedWithRock(*this)); // tests the new position, making sure it does not spawn on another round entity
}

float RoundEntity::GetRadius() const
{ 
	return ((float)GetWidth() / 2.0f); 
} 
void RoundEntity::SetMinDistance(float minDistance)
{
	assert(minDistance >= 0 && "Minimum distance cannot be negative");
	m_minDistance = minDistance;
}
void RoundEntity::SetRandomSpawnTimer(float minSpawnTime)
{
	assert(minSpawnTime >= 0 && "Cannot have negative minimum spawn time");
	assert(m_spawnInterval >= 1 && "Cannot have spawn interval less than 1");
	// minSpawn time is for rocks that are blocking to have a chance to get out of the way
	m_spawnCounter = minSpawnTime + (float) (rand()%m_spawnInterval); 
}
void RoundEntity::SetSpawnCounter(float spawnCounter)
{
	m_spawnCounter = spawnCounter;
}
float RoundEntity::GetSpawnCounter() const
{
	return m_spawnCounter;
}
void RoundEntity::SetSpawnInterval(int interval)
{
	m_spawnInterval = interval;
}