#include "ScreenManager.h"
#include "GraphicsSystem.h"
#include "BaseScreen.h"
#include "LogoScreen.h"
#include "MenuScreen.h"
#include "InstructionScreen.h"
#include "CreditScreen.h"
#include "StoryScreen.h"
#include "Globals.h" // FontSize

ScreenManager* ScreenManager::ms_screenManager = NULL; //init value on static instance handle

void ScreenManager::ShowScreen(UI_SCREEN screenToShow)
{
	m_uiScreens[screenToShow]->Draw();
}

ScreenManager* ScreenManager::GetSingleton() 
{
	if (ms_screenManager == NULL) 
	{
		ms_screenManager = new ScreenManager();
	}
	return ms_screenManager;
}

ScreenManager::ScreenManager()
{
	// screen are added in this way to make sure the screens are on their right positions
	// and later can be accessed using their m_uiScreens.at(UI_SCREEN)
	// no relocation nor moving should occur 
	// if the screens had been more resource-demanding, they would be loaded/unloaded depending on how close they are to be used (i.e. the new current or possible next screen)
	
	m_uiScreens.reserve(NR_OF_SCREENS); // trims capacity. linear complexity
	m_uiScreens.resize(NR_OF_SCREENS); // fills up with null pointers. linear complexity
	m_uiScreens.at(SCREEN_LOGO) = new LogoScreen();
	m_uiScreens.at(SCREEN_MENU) = new MenuScreen();
	m_uiScreens.at(SCREEN_INSTRUCTIONS) = new InstructionScreen();
	m_uiScreens.at(SCREEN_CREDITS) = new CreditScreen();
	m_uiScreens.at(SCREEN_STORY) = new StoryScreen();
}


ScreenManager::~ScreenManager()
{
	while(!m_uiScreens.empty())
	{
		delete m_uiScreens.back();
		m_uiScreens.pop_back();
	}		
}

void ScreenManager::Destroy()
{
	printf("INFO: Destroying ScreenManager\n");                                                 
	if(ms_screenManager != NULL)
	{
		delete ms_screenManager;
		ms_screenManager = NULL;
	}
}

void ScreenManager::ShowTextOnScreen(const char* text)
{
	// The right subsystem will throw error if texts are null or empty, no need to do it here
	
	float offset = 60.0f;
	GraphicsSystem::GetSingleton()->ClearScreenBufferWithBlack();
	GraphicsSystem::GetSingleton()->DrawTextOntoScreenBuffer(text, (float) SCREEN_HEIGHT / 2.0f - offset, CENTER, BLUE_TEXT_R_VALUE, BLUE_TEXT_G_VALUE, BLUE_TEXT_B_VALUE, FONT_SIZE_BIG, 1.0f);	
	GraphicsSystem::GetSingleton()->ShowBufferedScreenOnDisplay();
}
void ScreenManager::ShowTextOnScreen(const char* text1, const char* text2, bool bIsTransparentBackground)
{	
	// The right subsystem will throw error if texts are null or empty, no need to do it here

	if (!bIsTransparentBackground)
		GraphicsSystem::GetSingleton()->ClearScreenBufferWithBlack();

	float text1PosY = 180.0f;
	float text2PosY = 250.0f;

	GraphicsSystem::GetSingleton()->DrawTextOntoScreenBuffer(text1, text1PosY, CENTER, BLUE_TEXT_R_VALUE, BLUE_TEXT_G_VALUE, BLUE_TEXT_B_VALUE, FONT_SIZE_BIG, 1.0f);	
	GraphicsSystem::GetSingleton()->DrawTextOntoScreenBuffer(text2, text2PosY, CENTER, BLUE_TEXT_R_VALUE, BLUE_TEXT_G_VALUE, BLUE_TEXT_B_VALUE, FONT_SIZE_MEDIUM, 1.0f);	

	GraphicsSystem::GetSingleton()->ShowBufferedScreenOnDisplay();
}
