#pragma once //microsoft-specific

#include <vector>

// forward declaration to speed up compilation
class BaseScreen;

// Do not change order of these. Used to identify which screen should be shown
enum UI_SCREEN { 
		SCREEN_LOGO, 
		SCREEN_MENU, 
		SCREEN_INSTRUCTIONS, 
		SCREEN_CREDITS, 
		SCREEN_STORY, 
		NR_OF_SCREENS 
};

enum { 
		BLUE_TEXT_R_VALUE = 105, 
		BLUE_TEXT_G_VALUE = 155,
		BLUE_TEXT_B_VALUE = 255
};

/**
* Owns and contains functions for showing pre-set screens.
* Some screens are animations if they are called in loops.
* This class is a Singleton since there should only be one,
* globally accessible.
* Also has functions to show custom texts onscreen without
* need for a separate screen instance
*/
class ScreenManager
{
public:
	static 						ScreenManager* GetSingleton();

	// Shows the UIscreen specified
	void 						ShowScreen(UI_SCREEN screenToShow);

	// Shows text on screen. Enter only text, all other settings pre-set.
	void 						ShowTextOnScreen(const char* text);

	// Shows texts on screen. Enter only texts and opacity, all other settings pre-set.
	// Texts MUST contain at least one char.
	void 						ShowTextOnScreen(const char* text1, const char* text2, bool bIsTransparentBackground);

	// Call by same class that made first call. 
	// C++ deletes static objects auto at program termination, but it's done here manually
	// as good practice
	static void 				Destroy();
	

private:
	// Avoid multiple instantiation.
								ScreenManager();

	// Encapsulated to only be used by Destroy() in order to delete in a consistent way.
								~ScreenManager(); 

								ScreenManager(const ScreenManager &); // no copy
								ScreenManager &operator = (const ScreenManager &); // no assign

	std::vector<BaseScreen*>	m_uiScreens;
	static ScreenManager*		ms_screenManager; // static single instance
};

