#pragma once //microsoft-specific

/** Configuration file for Sound resources, divided into music and sfx 
* 
* An X-macro is then used to map the resource (enum) with its path string. 
* 
* Macros can be blunt and overwrite anything, don't have types, ignore scoping, 
* be hard to debug etc but this one is neat and I can stick to unique naming.
* 
* I consider it as good practice that the classes request resource by this ENUM and can't just type anything at any time. 
* Mistakes are usually found early at compile-time.
*/

// Mapping music ids (enum) with respective path. Keep in order! 
// Not really using the enums but they visualize the use and need for order well
#define MUSIC_ID_AND_PATH_TABLE \
X(LEVEL_1, "resources/sounds/music/Oliver Smith - Cadence (Original Mix).ogg") \
X(LEVEL_2, "resources/sounds/music/Nuera - Green Cape Sunset.ogg") \
X(LEVEL_3, "resources/sounds/music/Lost Shade - I Know What You Feel (Original Mix).ogg") \
X(LEVEL_4, "resources/sounds/music/Alpha 9 - Come Home (Original Mix).ogg") \
X(LEVEL_5, "resources/sounds/music/009 Sound System - Trinity.ogg") \
X(LEVEL_6, "resources/sounds/music/Nuera - Green Cape Sunset (continued).ogg")


// Mapping sound effects ids (enum) with belonging paths
// Note that each sound listed will create a separate channel
#define SFX_ID_AND_PATH_TABLE \
X(SFX_ID_BONUS_LIFE,"resources/sounds/sfx/bonus_life.ogg") \
X(SFX_ID_HIT,"resources/sounds/sfx/hit.ogg")


// X-macro that maps enums for sound resources together with their paths as strings
// Defining the enums below. The second part, belonging path definitions, can be found in SoundSystem.
#define X(a, b) a,

	enum SFX_ID {
	  SFX_ID_AND_PATH_TABLE
	};

	enum MUSIC_ID {
	  MUSIC_ID_AND_PATH_TABLE
	};

#undef X
