#pragma once //microsoft-specific

class ImageSprite; // forward declaration speeds up compilation

/**
* Represents a Starfield. 
* Has three layers of stars in order to create a feeling of 3D-depht in a 2D game:
* Front stars, moving fast and pulsating, large size
* Middle stars, moving fast, medium size
* Background stars, moving slow, small size
* 
* The depth feeling comes from when "closer" (larger) stars moves past "distant" (smaller) ones.
*
* The pulsating front stars layer also uses a special method to create a twinkling effect. 
* There are actually three image sprites making up this layer. 
* The stars in the layer move with the same speed as the middle stars and look
* like they are just extra intensive stars that are part of the middle layer field.
*
* It is toggled which one of the three front star imagesprites is used,
* so it's not the same stars twinkling all the time. 
* The current imagesprite pulsate from high to low opacity and back, 
* and then keep a short pause before starting to pulsate the next imagesprite.
*
* If Update() is called regularily, the starfield will be side-scrolling and pulsating.
*/
class StarField
{
public:
								StarField();
								~StarField();
	void 						Draw();
	void 						Update(float elapsedTime);

private:
	enum { 
								NUMBER_OF_GLOWING_STARFIELDS = 3 
	};

	// Avoid assign of this class, it should not be used in that way
	StarField& 					operator=(const StarField& );

	ImageSprite&				m_backgroundStarsRef;
	ImageSprite&				m_starFieldRef;
	ImageSprite*				m_glowingStars[NUMBER_OF_GLOWING_STARFIELDS];
	float 						m_glowingStarsCounter;
	float 						m_glowingStarsFullCycleCounterMax;
};

