#include "StateMachine.h"
#include "StateFactory.h"
#include "BaseState.h"
#include "Input.h"
#include <vector>
#include <stdio.h>

#define NULL 0

StateMachine::StateMachine(Timer& timerRef, Input& inputRef):
m_timerRef(timerRef),
m_inputRef(inputRef)
{
	m_stateFactory = new StateFactory();
	m_bShouldQuit = false;
	m_currentState = NULL;
}

StateMachine::~StateMachine()
{
	printf("INFO: Deallocating StateMachine resources\n");

	if (m_currentState != NULL)
	{
		delete m_currentState;
		m_currentState = NULL;
	}

	delete m_stateFactory;
	m_stateFactory = NULL;

}

void StateMachine::GoToNewState(StateKey state) 
{ 
	// If there is no state or the state is other than the one called to go to, then go there
	if (!m_currentState || !m_currentState->GetStateKey() || m_currentState->GetStateKey() != state)
	{
		if (m_currentState) // clean up old state
		{
			m_currentState->EndState();
			delete m_currentState;
		}

		m_currentState = &m_stateFactory->GetNewState(state, m_timerRef);
		m_currentState->BeginState();
	}
		
}
void StateMachine::RunMainLoop()
{
	printf("INFO: Entering StateMachine main loop\n");

	StateKey nextState = STATE_INTRO;
	std::vector<Action*> actions;

	do
	{
		GoToNewState(nextState);
		nextState = STATE_NONE;

		while(!ShouldQuit() && nextState == STATE_NONE)
		{
			actions = m_inputRef.GetNewActions();

			for (size_t i = 0; i < actions.size(); i++)
			{
				if (*(actions[i]) == A_QUIT)
					Quit(); // stop looping next time
			}

			m_currentState->Update(actions);
			
			nextState = m_currentState->DesiredStateChange(); 

			while (!actions.empty())
			{
				delete actions.back();
				actions.pop_back();
			}
			
		}
	}
	while(!ShouldQuit());

	m_currentState->EndState();

	printf("INFO: Exiting StateMachine main loop\n"); 

}

bool StateMachine::ShouldQuit() const
{
	return m_bShouldQuit;
}

void StateMachine::Quit()
{
	m_bShouldQuit = true;
}
