#pragma once //microsoft-specific

#include "Globals.h" // StateKey

// forward declaration to speed up compilation
class StateFactory;
class BaseState;
class Timer;
class Input;

/**
 * Engine driving the game. Contains the main loop. Takes input and updates current state with it. Handles transitions.
 **/
class StateMachine
{
public:
								StateMachine(Timer& timerRef, Input& inputRef);
								~StateMachine();

	// Main loop running the whole game.
	// Takes input and updates current state with it. Handles transitions between states. 
	void 						RunMainLoop();

private:

	// Ends and deallocates the current state and creates and begins a new
	void 						GoToNewState(StateKey state);

	// Signals that the game should shut down
	bool 						ShouldQuit() const;

	// Shut the game down on next loop
	void 						Quit();

	// Avoid assign of this class, it should not be used in that way
	StateMachine&				operator=(const StateMachine& ); 

	BaseState*					m_currentState;
	StateFactory*				m_stateFactory;
	Input&						m_inputRef;
	Timer&						m_timerRef;
	bool						m_bShouldQuit;
};
