#pragma once //microsoft-specific

#include "Globals.h" // XAlignment, FontSize
#include "SDL_ttf.h" // TTF_Font	
#include "SDL_opengl.h" // Uint8, GLfloat
#include <vector>
#include "TextConfig.h" // TEXT_ID

/**
* System for generating text graphics.
* Always call Init() first to initialize and load fonts.
* This should be at start of program.
* Has functions for drawing custom text to screen buffer. 
*
* Only one instance should exist of this class but it should NOT be globally available, instead only accessed 
* through the GraphicsSystem facade (which is why this class is not a singleton). 
* There could instead be argues that only GraphicsSystem should be a friend class that could access 
* and create an instance of this class, but that would break encapsulation and open up access 
* also to other private functions and variables to GraphicsSystem (functions it should not be able to use)
*/
class TextSubSystem
{
public:

	// Instantiates
								TextSubSystem();

	// Closes fonts and cleans up allocated memory
								~TextSubSystem();

	// Initializes ttf and fonts. Call at start of game.
	// Returns false if failed to init TTF or failed to find and open fonts
	bool 						Init();

	// Draws given text to screen buffer with specified settings. 
	void 						DrawTextOntoScreenBuffer(const char *textToDisplay, GLfloat& posY, XAlignment alignX, Uint8& textRed, Uint8& textGreen, Uint8& textBlue, FontSize fontSize, float& opacity);

	// Returns a vector with text lines from specified file. Will return an "missing text" placeholder in case of fail.
	// Internal references is kept and memory is deallocated automatically upon the destruction of textsubsystem.
	std::vector<const char*>	ReadTextFromFile(TEXT_ID id);

private:

	// Inits TTF and loads pre-set fonts with specified sizes
	// Returns false if fonts could not be found or opened
	bool 						LoadFonts();

	// Closes fonts
	void 						UnloadFonts();	

	TTF_Font* 					m_mediumFont;
	TTF_Font* 					m_bigFont;
	TTF_Font* 					m_smallFont;

	// coloring masks
	Uint32 						m_mask_r;
	Uint32 						m_mask_g;
	Uint32 						m_mask_b;
	Uint32 						m_mask_a;  
	bool 						m_bColorIsRGBA; // flag for alpha mode available

	std::vector<std::vector<const char*>> m_allocated_texts; // internal references all dynamically allocated texts, released upon destrution
};